// Copyright (c) 2013, Thomas Goyne <plorkyeran@aegisub.org>
//
// Permission to use, copy, modify, and distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
// Aegisub Project http://www.aegisub.org/

#include "libaegisub/log.h"

#include <libaegisub/path.h>

#include <libaegisub/exception.h>
#include <libaegisub/util_osx.h>

#include <boost/filesystem/operations.hpp>
#include <pwd.h>

namespace {
#ifndef __APPLE__
std::string home_dir()
{
    const char *env = getenv("HOME");
    if (env) return env;

    if ((env = getenv("USER")) || (env = getenv("LOGNAME"))) {
        if (passwd *user_info = getpwnam(env))
            return user_info->pw_dir;
    }

    throw agi::EnvironmentError("Could not get home directory. Make sure HOME is set.");
}
#endif
}

#include <iostream>

namespace agi {
void Path::FillPlatformSpecificPaths()
{
#ifndef __APPLE__
    agi::fs::path home = home_dir();
    SetToken("?user", home / ".aegisub");
    SetToken("?local", home / ".aegisub");
#ifdef AEGISUB_LINUX
    char *P_EXEC = getenv("P_EXEC");
    if (P_EXEC && P_EXEC[0] != '\0' && std::string(P_EXEC).rfind("/tmp/.mount_") == 0) {
        /* Craft the P_DATA with the P_EXEC because we are in an AppImage The
         * string should be /tmp/.mount_(.*)/usr/bin/aegisub and need to be
         * transformed to /tmp/.mount_(.*)/usr/share/aegisub/ */
        agi::fs::path p_exec(P_EXEC);
        std::string p_data = (p_exec.parent_path().parent_path() / "share/aegisub").string();
        SetToken("?data", p_data);
    }
    else {
        SetToken("?data", P_DATA);
    }
#else
    SetToken("?data", P_DATA);
#endif /* AEGISUB_LINUX */
    SetToken("?dictionary", "/usr/share/hunspell");
#else /* Now __APPLE__ is defined */
    agi::fs::path app_support = agi::util::GetApplicationSupportDirectory();
    SetToken("?user", app_support / "Aegisub");
    SetToken("?local", app_support / "Aegisub");
    SetToken("?data", agi::util::GetBundleSharedSupportDirectory());
    SetToken("?dictionary", agi::util::GetBundleSharedSupportDirectory() + "/dictionaries");
#endif /* __APPLE__ */
    SetToken("?temp", boost::filesystem::temp_directory_path());
}

}
